@@ -78,6 +78,11 @@ sub find_profile($;%) {
    my ($name, %kv) = @_;
 
    +{
+      monitor_timeout  => 30,
+      connect_interval => 2,
+      data_format      => [qw(json storable)], # framing types we offer and accept, in order of preference
+      auth_offer       => [qw(tls_md6_64_256 hmac_md6_64_256)], # what we will send
+      auth_accept      => [qw(tls_md6_64_256 hmac_md6_64_256 tls_anon cleartext)], # what we accept
       %kv,
       _find_profile $name,
    }
